#define MAKE_UINT(a,b,c,d) ((d<<24)|(c<<16)|(b<<8)|(a))

typedef unsigned int uint32;
typedef struct  {
    uint32 dwSize;
    uint32 dwFlags;
    uint32 dwFourCC;
    uint32 dwRGBBitCount;
    uint32 dwRBitMask, dwGBitMask, dwBBitMask;
    uint32 dwRGBAlphaBitMask;
} DDPIXELFORMAT;

typedef struct  {
    uint32 dwCaps1;
    uint32 dwCaps2;
    uint32 Reserved[2];
} DDSCAPS2;

#define DDSD_MIPMAPCOUNT        0x00020000l

#define DDPF_RGB                0x00000040l
#define DDPF_FOURCC             0x00000004l
#define DDPF_ALPHAPIXELS        0x00000001l


typedef struct  {
    uint32 dwSize;
    uint32 dwFlags;
    uint32 dwHeight;
    uint32 dwWidth;
    uint32 dwPitchOrLinearSize;
    uint32 dwDepth;
    uint32 dwMipMapCount;
    uint32 dwReserved1[11];
    DDPIXELFORMAT ddpfPixelFormat;
    DDSCAPS2 ddsCaps;
    uint32 dwReserved2;
} DDSURFACEDESC2;

void convertToDDS(const char *data, unsigned int size, char*&newData, unsigned int& newSize)
{
	int width, height, comps;
	void* pixData = stbi_load_from_memory( (unsigned char *)data, size, &width, &height, &comps, 4 );
	if( pixData == 0x0 || (comps!=3&&comps!=4) )
	{
		newData = NULL;
		newSize = 0;
		return;
	}

	//create DDS
	newSize = sizeof(uint32) + sizeof(DDSURFACEDESC2) + width * height * 4;
	newData = new char[newSize];

	*((uint32*)newData)= MAKE_UINT('D','D','S',' ');

	DDSURFACEDESC2 *desc = (DDSURFACEDESC2*)(newData + sizeof(uint32));
	memset(desc, 0, sizeof(DDSURFACEDESC2));
	desc->dwSize = sizeof(DDSURFACEDESC2);
	desc->dwFlags = 0x1007;
    desc->dwHeight = height;
    desc->dwWidth = width;

	if (comps == 3)
	{	//RGB
		desc->ddpfPixelFormat.dwFlags = DDPF_RGB;
		desc->ddpfPixelFormat.dwRGBBitCount = 32;
		desc->ddpfPixelFormat.dwRBitMask = 0x000000ff;
		desc->ddpfPixelFormat.dwGBitMask = 0x0000ff00;
		desc->ddpfPixelFormat.dwBBitMask = 0x00ff0000;
	}
	else //comps == 4
	{
		desc->ddpfPixelFormat.dwFlags = (DDPF_RGB|DDPF_ALPHAPIXELS);
		desc->ddpfPixelFormat.dwRGBBitCount = 32;
		desc->ddpfPixelFormat.dwRBitMask = 0x000000ff;
		desc->ddpfPixelFormat.dwGBitMask = 0x0000ff00;
		desc->ddpfPixelFormat.dwBBitMask = 0x00ff0000;
		desc->ddpfPixelFormat.dwRGBAlphaBitMask = 0xff000000;
	}

	//copy image data
	memcpy(newData + sizeof(uint32) + sizeof(DDSURFACEDESC2), pixData, width * height * 4);

	stbi_image_free( pixData );
}
